"""
Bili_UAS.cli.user_cli

This module provides user command line interface configuration.
"""


import tyro
from typing import Union, Literal
from dataclasses import dataclass


@dataclass
class BiliUserConfigUpdate(object):
    """

    """
    name: Union[str, None] = None
    """username, either name or uid must be filled in"""
    uid: Union[str, None] = None
    """user uid, either name or uid must be filled in"""


@dataclass
class BiliUserConfigAddress(object):
    """

    """
    name: Union[str, None] = None
    """username, either name or uid must be filled in"""
    uid: Union[str, None] = None
    """user uid, either name or uid must be filled in"""
    mode: Literal[1, 2] = 1
    """address process mode, 1 for send, 2 for receive"""


mode_configs: dict[str, Union[BiliUserConfigUpdate, BiliUserConfigAddress]] = {}

descriptions: dict[str, str] = {
    "update": "Update user fan number, guard number, charging number.",
    "address": "Count the guard's address."
}

mode_configs["update"] = BiliUserConfigUpdate()
mode_configs["address"] = BiliUserConfigAddress()

UserConfigUnion = tyro.conf.SuppressFixed[
    tyro.conf.FlagConversionOff[
        tyro.extras.subcommand_type_from_defaults(defaults=mode_configs, descriptions=descriptions)
    ]
]



"""
Bili_UAS.bili_live

This module provides a command line interface for monitoring the live broadcast room and processing data
"""


from __future__ import annotations
from Bili_UAS.utils import config_utils as ucu
from Bili_UAS.utils import live_utils as ulu, user_utils as uuu
from typing import Union
from Bili_UAS.writer import log_writer as wlw, abnormal_monitor as wam
import os
from bilibili_api import sync
from Bili_UAS.scripts import log_in as sli
from Bili_UAS.cli import live_cli as clc
import tyro
import matplotlib.pyplot as plt
from numpy import typing as npt


def sync_tyro_main(config: Union[clc.BiliLiveConfigAuto, clc.BiliLiveConfigMonitor, clc.BiliLiveConfigProcess]) -> None:
    """
    Main function for tyro command-line interface.

    Args:
        config: configuration
    """
    language: str = ucu.load_language_from_txt()
    work_dir: str = sync(ucu.load_work_dir_from_txt())
    log_output: str = os.path.join(work_dir, "log")
    log_file: str = os.path.join(log_output, "live_log.txt")

    file_handler: wlw.Handler = wlw.Handler("file")
    file_handler.set_level("WARNING", "ERROR")
    file_handler.set_file(log_file)

    sys_handler: wlw.Handler = wlw.Handler("sys")
    sys_handler.set_level("INFO", "WARNING")

    log: wlw.Logger = wlw.Logger()
    log.add_config(file_handler)
    log.add_config(sys_handler)

    credential = sync(sli.load_credential_from_json(log_file))
    if credential is not None:
        credential = sync(sli.refresh_credential(credential, log_file))

    if isinstance(config, clc.BiliLiveConfigAuto):
        if language == "en":
            log.warning("The setting mode is 'auto', and in this mode, 'auto_disconnect', 'danmu_disconnect', and "
                        "'robust' are set to true, and data processing is automatically performed after disconnecting "
                        "the live streaming connection!")
        else:
            log.warning("设置模式为 'auto'，在此模式下，'auto_disconnect'、'danmu_disconnect'、'robust' 被设置为True，"
                        "且在断开直播连接后自动进行数据处理！")

        config.auto_disconnect = True
        config.danmu_disconnect = True
        config.robust = True

        if config.user_id is None:
            if config.live_id is None:
                if language == "en":
                    raise wam.ParameterInputError("user_id and live_id must be entered either!")
                else:
                    raise wam.ParameterInputError("user_id 和 live_id 必须输入其中之一！")

        if config.user_id is not None:
            user = uuu.BiliUser(uid=config.user_id, log=log_file, work_dir=work_dir, credential=credential)
            config.live_id = user.room_id

        live_monitor = ulu.BiliLiveMonitor(config.live_id, log_file, work_dir, config.max_retry,
                                           config.retry_after, credential)
        sync(live_monitor.load_danmu_mark())
        mask: npt.NDArray = plt.imread(config.mask)

        sync(live_monitor.monitor(config.save_all_danmu, config.danmu_disconnect, config.auto_disconnect))
        live_process = ulu.BiliLiveProcess(log_file, live_monitor.work_dir)
        sync(live_process.analysis(config.revenue_interval, config.danmu_interval, config.robust,
                                   config.robust_interval, mask))

        while config.forever:
            if language == "en":
                log.warning("Long connecting live room. To exit the program, please use ctrl + c.")
            else:
                log.warning("设置为长连直播间，要退出程序，请使用ctrl + c。")
            sync(live_monitor.monitor(config.save_all_danmu, config.danmu_disconnect, config.auto_disconnect))
            live_process = ulu.BiliLiveProcess(log_file, live_monitor.work_dir)
            sync(live_process.analysis(config.revenue_interval, config.danmu_interval, config.robust,
                                       config.robust_interval, mask))

    elif isinstance(config, clc.BiliLiveConfigMonitor):
        if language == "en":
            log.warning("Set the mode to 'monitor', and in this mode, only data monitoring "
                        "will be performed without data processing!")
        else:
            log.warning("设置模式为 'monitor'，在此模式下，只进行数据监控，不进行数据处理！")

        if config.user_id is None:
            if config.live_id is None:
                if language == "en":
                    raise wam.ParameterInputError("user_id and live_id must be entered either!")
                else:
                    raise wam.ParameterInputError("user_id 和 live_id 必须输入其中之一！")

        if config.user_id is not None:
            user = uuu.BiliUser(uid=config.user_id, log=log_file, work_dir=work_dir, credential=credential)
            config.live_id = user.room_id

        live_monitor = ulu.BiliLiveMonitor(config.live_id, log_file, work_dir, config.max_retry,
                                           config.retry_after, credential)
        sync(live_monitor.load_danmu_mark())

        sync(live_monitor.monitor(config.save_all_danmu, config.danmu_disconnect, config.auto_disconnect))
        while config.forever:
            if language == "en":
                log.warning("Long connecting live room. To exit the program, please use ctrl + c.")
            else:
                log.warning("设置为长连直播间，要退出程序，请使用ctrl + c。")
            sync(live_monitor.monitor(config.save_all_danmu, config.danmu_disconnect, config.auto_disconnect))

    elif isinstance(config, clc.BiliLiveConfigProcess):
        if language == "en":
            log.warning("Set the mode to 'process', and in this mode, a data folder needs to be specified!")
        else:
            log.warning("设置模式为 'process'，在此模式下，需要指定一个数据文件夹！")

        if config.data_dir is None:
            if language == "en":
                raise wam.ParameterInputError("No data folder specified!")
            else:
                raise wam.ParameterInputError("没有指定数据文件夹！")

        if config.mask is not None:
            mask: npt.NDArray = plt.imread(config.mask)
        else:
            mask = None
        live_process = ulu.BiliLiveProcess(log_file, config.data_dir)
        sync(live_process.analysis(config.revenue_interval, config.danmu_interval, config.robust,
                                   config.robust_interval, mask))


def tyro_cli() -> None:
    """
    Tyro command line interface.
    """
    tyro.extras.set_accent_color("bright_yellow")
    sync_tyro_main(
        tyro.cli(clc.LiveConfigUnion, description="Monitor and process live broadcast room data.")
    )


if __name__ == "__main__":
    tyro_cli()



"""
Bili_UAS.utils.video_utils

This module provides the BiliVideo class, which is used to get video information, danmu, replies and tags.
"""


from __future__ import annotations
import numpy as np
from .utils import BiliVideoReply, BiliVideoDanmu, BiliVideoTag
from .config_utils import load_language_from_txt, load_ffmpeg_path_from_txt
from Bili_UAS.writer import log_writer as wlw
import time
import copy
import re
from bilibili_api import Credential, video as bav, sync, Danmaku, comment as bac, HEADERS
import pandas as pd
from pandas import DataFrame
import os
from typing import Union
import httpx
import enum


language: str = load_language_from_txt()


@wlw.async_separate()
async def _download_video_from_url(video_url: str,
                                   output_file: str,
                                   video_pid: int,
                                   log: wlw.Logger,
                                   prompt_prefix: str) -> None:
    """
    Download video through url.

    Args:
        video_url: download url for video
        output_file: the path to save the video
        video_pid: pid of video
        log: the log class
    """
    if language == "en":
        log.info(f"{video_pid} {prompt_prefix} downloading...")
    else:
        log.info(f"{video_pid} {prompt_prefix} 正在下载...")
    async with httpx.AsyncClient(headers=HEADERS) as sess:
        resp = await sess.get(video_url)
        length = resp.headers.get('content-length')

        with open(output_file, 'wb') as f:
            process = 0
            for chunk in resp.iter_bytes(1024):
                if not chunk:
                    break
                process += len(chunk)
                if language == "en":
                    log.info(f"{video_pid} {prompt_prefix} downloading... {process} / {length}")
                else:
                    log.info(f"{video_pid} {prompt_prefix} 正在下载... {process} / {length}")
                f.write(chunk)

    if language == "en":
        log.info(f"{video_pid} {prompt_prefix} download successfully.")
    else:
        log.info(f"{video_pid} {prompt_prefix} 下载成功.")


class VideoDownloadMode(enum.Enum):
    """
    Video Download Type Enumeration Class
    """
    VIDEO = 1
    AUDIO = 2


class BiliVideo(bav.Video):
    """
    Bilibili Video Class.
    """

    def __init__(self,
                 log: str,
                 work_dir: str,
                 credential: Union[Credential, None] = None,
                 aid: Union[int, None] = None,
                 bvid: Union[str, None] = None) -> None:
        """
        Either aid or bvid must be filled in.

        Args:
            aid: video aid
            bvid: video bvid
            log: the log file
            work_dir: working directory
            credential: logon credentials
        """
        super().__init__(bvid=bvid, aid=aid, credential=credential)
        self.aid: int = self.get_aid()
        self.bvid: str = self.get_bvid()
        self.p_cid: list[int] = []
        self.p_time: list[int] = []

        self.publish_time: Union[int, None] = None
        self.total_time: Union[int, None] = None
        self.view: Union[int, None] = None
        self.like: Union[int, None] = None
        self.coin: Union[int, None] = None
        self.favorite: Union[int, None] = None
        self.share: Union[int, None] = None
        self.history_rank: Union[int, None] = None
        self.reply_num: Union[int, None] = None
        self.danmu_num: Union[int, None] = None
        self.copyright: Union[int, None] = None
        self.reprint_sign: Union[int, None] = None
        self.up_uid: Union[int, None] = None
        self.tag_use_mean: Union[int, None] = None
        self.tag_use_min: Union[int, None] = None
        self.tag_use_max: Union[int, None] = None
        self.tag_follow_mean: Union[int, None] = None
        self.tag_follow_min: Union[int, None] = None
        self.tag_follow_max: Union[int, None] = None

        self.replies: list[BiliVideoReply] = []
        self.robust_replies: list[BiliVideoReply] = []
        self.danmu: list[BiliVideoDanmu] = []
        self.tags: list[BiliVideoTag] = []

        self.work_dir: Union[str, None] = None
        self.info_excel_file: Union[str, None] = None
        self.info_excel: Union[DataFrame, None] = None

        self.log_file: str = log
        self.log: Union[wlw.Logger, None] = None
        self.__set_log()
        self.__p_video_init()
        self.__load_work_dir(work_dir)

    def __set_log(self) -> None:
        """
        Set up logs.
        """
        file_handler: wlw.Handler = wlw.Handler("file")
        file_handler.set_level("WARNING", "ERROR")
        file_handler.set_file(self.log_file)

        sys_handler: wlw.Handler = wlw.Handler("sys")
        sys_handler.set_level("INFO", "WARNING")

        self.log: wlw.Logger = wlw.Logger()
        self.log.add_config(file_handler)
        self.log.add_config(sys_handler)

    def __p_video_init(self) -> None:
        """
        Obtain sub video information, including id and video time.
        """
        p_info: list[dict] = sync(self.get_pages())
        if p_info:
            for page in p_info:
                self.p_cid.append(page['cid'])
                self.p_time.append(page['duration'])
        else:
            if language == "en":
                self.log.warning("Failed to obtain sub video ID, which may affect subsequent operations!")
            else:
                self.log.warning("未获取到分P视频ID, 这可能会影响后续操作!")

    def __load_work_dir(self, work_dir: str) -> None:
        """
        Load the working directory.

        Args:
            work_dir: working directory
        """
        video_output_dir: str = os.path.join(work_dir, "video_output")
        self.work_dir: str = os.path.join(video_output_dir, self.bvid)
        if not os.path.exists(self.work_dir):
            os.mkdir(self.work_dir)

    @wlw.async_separate()
    async def video_info_statistics(self) -> None:
        """
        Obtain current video data information.
        """
        if language == "en":
            self.log.info(f"Start acquiring video information for {self.bvid}...")
        else:
            self.log.info(f"开始获取 {self.bvid} 的视频信息...")

        video_info: dict = await self.get_info()
        self.publish_time = video_info['pubdate']
        self.total_time = video_info['duration']
        self.view = video_info['stat']['view']
        self.like = video_info['stat']['like']
        self.coin = video_info['stat']['coin']
        self.favorite = video_info['stat']['favorite']
        self.share = video_info['stat']['share']
        self.history_rank = video_info['stat']['his_rank']
        self.reply_num = video_info['stat']['reply']
        self.danmu_num = video_info['stat']['danmaku']
        self.copyright = video_info['copyright']  # copyright: copyright mark, 1: homemade, 2: reprint
        self.up_uid = video_info['owner']['mid']

        video_stst: dict = await self.get_stat()
        self.reprint_sign = video_stst[
            'no_reprint']  # reprint_sign: prohibition of reprinting sign, 0: none, 1: prohibition

        if language == "en":
            self.log.info(f"Video information acquisition completed for {self.bvid}.")
        else:
            self.log.info(f"{self.bvid} 的视频信息获取完成.")

    @wlw.async_separate()
    async def get_replies(self, sec: bool) -> None:
        """
        Obtain first level (second level) replies of videos.

        Args:
            sec: whether to obtain the second level reply
        """
        if language == "en":
            self.log.info(f"Start acquiring replies for {self.bvid}...")
        else:
            self.log.info(f"开始获取 {self.bvid} 的评论...")
        page: int = 1
        count: int = 0
        while True:
            if language == "en":
                self.log.info(f"Start acquiring page {page} of replies for {self.bvid}.")
            else:
                self.log.info(f"开始获取 {self.bvid} 第 {page} 页评论.")
            page_reply_info: dict = await bac.get_comments(self.aid, bac.CommentResourceType.VIDEO, page,
                                                           credential=self.credential)
            count += page_reply_info['page']['size']
            if page_reply_info['replies']:
                for r in page_reply_info['replies']:
                    reply: BiliVideoReply = BiliVideoReply(r, log=self.log_file)
                    self.replies.append(reply)
                    if sec:
                        if r['replies']:
                            for sub_r in r['replies']:
                                sub_reply: BiliVideoReply = BiliVideoReply(sub_r, log=self.log_file)
                                self.replies.append(sub_reply)
                page += 1
                time.sleep(0.2)
                if count >= page_reply_info['page']['count']:
                    break
            else:
                break
        if sec:
            if language == "en":
                self.log.info(
                    f"A total of {len(self.replies)} replies have been collected successfully. "
                    f"(With second level replies)")
            else:
                self.log.info(
                    f"一共成功获取了 {len(self.replies)} 条评论. (包含二级评论)")
        else:
            if language == "en":
                self.log.info(
                    f"A total of {len(self.replies)} replies have been collected successfully. "
                    f"(Without second level replies)")
            else:
                self.log.info(
                    f"一共成功获取了 {len(self.replies)} 条评论. (不包含二级评论)")

    @wlw.async_separate()
    async def get_danmu(self) -> None:
        """
        Obtain all current danmu in the video.
        """
        if language == "en":
            self.log.info(f"Start acquiring danmu for {self.bvid}...")
        else:
            self.log.info(f"开始获取 {self.bvid} 的弹幕...")
        if self.p_cid:
            for p_id in self.p_cid:
                if language == "en":
                    self.log.info(f"Start acquiring danmu for sub video: {p_id}...")
                else:
                    self.log.info(f"开始获取分P: {p_id} 的弹幕...")
                danmu_list_info: list[Danmaku] = await self.get_danmakus(cid=p_id)
                if danmu_list_info:
                    for danmu_info in danmu_list_info:
                        danmu: BiliVideoDanmu = BiliVideoDanmu(danmu_info, log=self.log_file)
                        self.danmu.append(danmu)
                time.sleep(0.2)
            if language == "en":
                self.log.info(f"A total of {len(self.danmu)} danmu have been collected successfully.")
            else:
                self.log.info(f"一共成功获取了 {len(self.danmu)} 条弹幕.")
        else:
            if language == "en":
                self.log.warning("The sub video id is missing, and the danmu cannot be obtained!")
            else:
                self.log.warning("缺少分P视频id, 无法获取弹幕!")

    @wlw.async_separate()
    async def reply_robust_process(self) -> None:
        """
        Robust processing of replies. Remove emoticon frame.
        """
        if language == "en":
            self.log.info(f"Start robust processing of replies for {self.bvid}...")
        else:
            self.log.info(f"开始处理 {self.bvid} 的评论内容...")
        if self.replies:
            for elem in self.replies:
                rb_reply: BiliVideoReply = copy.deepcopy(elem)
                rb_reply.content = re.sub(r"\[.*?]", ",", rb_reply.content)
                self.robust_replies.append(rb_reply)
            if language == "en":
                self.log.info(f"Robust processing of replies for {self.bvid} completed.")
            else:
                self.log.info(f"完成对 {self.bvid} 评论内容的处理.")
        else:
            if language == "en":
                self.log.warning("The reply is empty, and the robust processing cannot be performed!")
            else:
                self.log.warning("评论为空, 无法进行处理!")

    @wlw.async_separate()
    async def get_tag(self) -> None:
        """
        Obtain video tag information.
        """
        if language == "en":
            self.log.info(f"Start acquiring tags for {self.bvid}...")
        else:
            self.log.info(f"开始获取 {self.bvid} 的标签...")
        if self.p_cid:
            for p_id in self.p_cid:
                if language == "en":
                    self.log.info(f"Start acquiring tags for sub video: {p_id}.")
                else:
                    self.log.info(f"开始获取分P: {p_id} 的标签...")
                tag_info_list: list[dict] = await self.get_tags(cid=p_id)
                if tag_info_list:
                    for tag_info in tag_info_list:
                        tag: BiliVideoTag = BiliVideoTag(tag_info, log=self.log_file)
                        self.tags.append(tag)
                time.sleep(0.2)
            if len(self.tags) > 0:
                if language == "en":
                    self.log.info(f"A total of {len(self.tags)} tags have been collected successfully.")
                else:
                    self.log.info(f"一共成功获取了 {len(self.tags)} 个标签.")
            else:
                if language == "en":
                    self.log.warning(f"{self.bvid} did not add a tag, which may affect subsequent operations!")
                else:
                    self.log.warning(f"{self.bvid} 没有添加标签, 这可能会影响后续操作!")
        else:
            if language == "en":
                self.log.warning("The sub video id is missing, and the tag cannot be obtained!")
            else:
                self.log.warning("缺少分P视频id, 无法获取标签!")

    @wlw.async_separate()
    async def tag_process(self) -> None:
        """
        Calculate the mean, maximum and minimum values of video tag popularity.
        """
        if language == "en":
            self.log.info(f"Start processing tags for {self.bvid}...")
        else:
            self.log.info(f"开始处理 {self.bvid} 的标签...")
        tag_use: list[int] = []
        tag_follow: list[int] = []
        if self.tags:
            for elem in self.tags:
                tag_use.append(elem.use_num)
                tag_follow.append(elem.follow_num)
            self.tag_use_mean: float = sum(tag_use) / len(tag_use)
            self.tag_use_max: int = np.max(tag_use)
            self.tag_use_min: int = np.min(tag_use)
            self.tag_follow_mean: float = sum(tag_follow) / len(tag_follow)
            self.tag_follow_max: int = np.max(tag_follow)
            self.tag_follow_min: int = np.min(tag_follow)
            if language == "en":
                self.log.info(f"Processing tags for {self.bvid} completed.")
            else:
                self.log.info(f"完成对 {self.bvid} 标签的处理.")
        else:
            self.tag_follow_mean: float = -1
            self.tag_follow_max: int = -1
            self.tag_follow_min: int = -1
            self.tag_use_mean: float = -1
            self.tag_use_max: int = -1
            self.tag_use_min: int = -1
            if language == "en":
                self.log.warning("The tag is empty!")
            else:
                self.log.warning("标签为空!")

    async def __load_info_excel(self, excel_file: str) -> None:
        """
        Load the Excel file to save the video information.

        Args:
            excel_file: excel file path
        """
        if not os.path.exists(excel_file):
            temp_excel: DataFrame = pd.DataFrame()
            temp_excel.to_excel(excel_file)
        self.info_excel: DataFrame = pd.read_excel(excel_file)

    async def info_to_excel(self, excel_file: str) -> None:
        """
        Save video information to Excel file.

        Args:
            excel_file: excel file path
        """
        await self.__load_info_excel(excel_file)
        line: DataFrame = pd.DataFrame({"aid": self.aid,
                                        "bvid": self.bvid,
                                        "up_uid": self.up_uid,
                                        "publish_time": self.publish_time,
                                        "total_time": self.total_time,
                                        "view": self.view,
                                        "like": self.like,
                                        "coin": self.coin,
                                        "favorite": self.favorite,
                                        "share": self.share,
                                        "history_rank": self.history_rank,
                                        "reply_num": self.reply_num,
                                        "danmu_num": self.danmu_num,
                                        "copyright": self.copyright,
                                        "reprint_sign": self.reprint_sign,
                                        "tag_use_mean": self.tag_use_mean,
                                        "tag_use_max": self.tag_use_max,
                                        "tag_use_min": self.tag_use_min,
                                        "tag_follow_mean": self.tag_follow_mean,
                                        "tag_follow_max": self.tag_follow_max,
                                        "tag_follow_min": self.tag_follow_min},
                                       index=[0])
        pd.concat([self.info_excel, line], axis=0, ignore_index=True).to_excel(excel_file, index=False)
        if language == "en":
            self.log.info(f"Video information has been saved to {excel_file}.")
        else:
            self.log.info(f"视频信息已保存至 {excel_file}.")

    @wlw.async_separate()
    async def download(self, mode: VideoDownloadMode) -> None:
        """
        Download all videos or audio.

        Args:
            mode: 0 for downloading videos, 1 for downloading audio.
        """
        if self.p_cid:
            ffmpeg_path = await load_ffmpeg_path_from_txt()
            for pid in self.p_cid:
                p_url_info: dict = await self.get_download_url(cid=pid)
                detector = bav.VideoDownloadURLDataDetecter(data=p_url_info)
                streams = detector.detect_best_streams()

                if detector.check_flv_stream():
                    if mode == VideoDownloadMode.VIDEO:
                        temp_flv: str = os.path.join(self.work_dir, f"{pid}_flv_temp.flv")
                        mp4_video_out: str = os.path.join(self.work_dir, f"{pid}.mp4")

                        await _download_video_from_url(streams[0].url, temp_flv, pid, self.log, "flv video streaming")

                        if language == "en":
                            self.log.info("Converting video format...")
                        else:
                            self.log.info("正在转换视频格式...")
                        os.system(f"{ffmpeg_path} -i {temp_flv} {mp4_video_out}")

                        os.remove(temp_flv)
                        if language == "en":
                            self.log.info(f"{pid} video download successfully.")
                        else:
                            self.log.info(f"{pid} 视频下载成功.")

                    elif mode == VideoDownloadMode.AUDIO:
                        temp_flv: str = os.path.join(self.work_dir, f"{pid}_flv_temp.flv")
                        mp3_audio_out: str = os.path.join(self.work_dir, f"{pid}.mp3")

                        await _download_video_from_url(streams[0].url, temp_flv, pid, self.log, "flv video streaming")

                        if language == "en":
                            self.log.info("Converting audio format...")
                        else:
                            self.log.info("正在转换音频格式...")
                        os.system(f"{ffmpeg_path} -i {temp_flv} -vn -acodec copy {mp3_audio_out}")

                        os.remove(temp_flv)
                        if language == "en":
                            self.log.info(f"{pid} audio download successfully.")
                        else:
                            self.log.info(f"{pid} 音频下载成功.")
                else:
                    if mode == VideoDownloadMode.VIDEO:
                        temp_m4s_video: str = os.path.join(self.work_dir, f"{pid}_video_mp4_temp.m4s")
                        temp_m4s_audio: str = os.path.join(self.work_dir, f"{pid}_audio_mp4_temp.m4s")
                        mp4_video_out: str = os.path.join(self.work_dir, f"{pid}.mp4")

                        await _download_video_from_url(streams[0].url, temp_m4s_video, pid, self.log, "video streaming")
                        await _download_video_from_url(streams[1].url, temp_m4s_audio, pid, self.log, "audio streaming")

                        if language == "en":
                            self.log.info("Converting video format...")
                        else:
                            self.log.info("正在转换视频格式...")
                        os.system(f"{ffmpeg_path} -i {temp_m4s_video} -i {temp_m4s_audio} "
                                  f"-vcodec copy -acodec copy {mp4_video_out}")

                        os.remove(temp_m4s_video)
                        os.remove(temp_m4s_audio)
                        if language == "en":
                            self.log.info(f"{pid} video download successfully.")
                        else:
                            self.log.info(f"{pid} 视频下载成功.")

                    elif mode == VideoDownloadMode.AUDIO:
                        temp_m4s_audio: str = os.path.join(self.work_dir, f"{pid}_audio_mp4_temp.m4s")
                        mp3_audio_out: str = os.path.join(self.work_dir, f"{pid}.mp3")

                        await _download_video_from_url(streams[1].url, temp_m4s_audio, pid, self.log, "audio streaming")

                        if language == "en":
                            self.log.info("Converting audio format...")
                        else:
                            self.log.info("正在转换音频格式...")
                        os.system(f"{ffmpeg_path} -i {temp_m4s_audio} -acodec copy {mp3_audio_out}")

                        os.remove(temp_m4s_audio)
                        if language == "en":
                            self.log.info(f"{pid} audio download successfully.")
                        else:
                            self.log.info(f"{pid} 音频下载成功.")



"""
Bili_UAS.bili_video

This module provides a command line interface for downloading videos, audio or generating word image.
"""


from __future__ import annotations
import matplotlib.pyplot as plt
import tyro
from Bili_UAS.utils import config_utils as ucu, video_utils as uvu
from Bili_UAS.scripts import video as sv, log_in as sli
from Bili_UAS.cli import video_cli as cvc
from Bili_UAS.writer import log_writer as wlw, abnormal_monitor as wam
from bilibili_api import sync
import os
from typing import Union
from numpy import typing as npt


language: str = ucu.load_language_from_txt()


def sync_tyro_main(config: Union[cvc.BiliVideoConfigWordCloud, cvc.BiliVideoConfigDownload]) -> None:
    """
    Main function for tyro command-line interface.

    Args:
        config: configuration
    """
    work_dir: str = sync(ucu.load_work_dir_from_txt())
    log_output: str = os.path.join(work_dir, "log")
    log_file: str = os.path.join(log_output, "video_log.txt")

    file_handler: wlw.Handler = wlw.Handler("file")
    file_handler.set_level("WARNING", "ERROR")
    file_handler.set_file(log_file)

    sys_handler: wlw.Handler = wlw.Handler("sys")
    sys_handler.set_level("INFO", "WARNING")

    log: wlw.Logger = wlw.Logger()
    log.add_config(file_handler)
    log.add_config(sys_handler)

    credential = sync(sli.load_credential_from_json(log_file))
    if credential is not None:
        credential = sync(sli.refresh_credential(credential, log_file))

    if isinstance(config, cvc.BiliVideoConfigWordCloud):
        if config.video_id is None:
            if language == "en":
                raise wam.ParameterInputError("Video ID not entered!")
            else:
                raise wam.ParameterInputError("未输入视频ID！")
        wm = sv.WordCloudContent(config.mode)
        if config.mask is not None:
            word_cloud_mask: npt.NDArray = plt.imread(config.mask)
        else:
            word_cloud_mask = None
        sync(sv.word_cloud(config.video_id, credential, wm, config.sec, word_cloud_mask, log_file, work_dir))
    else:
        if config.video_id is None:
            if language == "en":
                raise wam.ParameterInputError("Video ID not entered!")
            else:
                raise wam.ParameterInputError("未输入视频ID！")
        dm = uvu.VideoDownloadMode(config.mode)
        if isinstance(config.video_id, int):
            video = uvu.BiliVideo(log=log_file, aid=config.video_id, credential=credential, work_dir=work_dir)
        else:
            video = uvu.BiliVideo(log=log_file, bvid=config.video_id, credential=credential, work_dir=work_dir)
        sync(video.download(dm))


def tyro_cli() -> None:
    """
    Tyro command line interface.
    """
    tyro.extras.set_accent_color("bright_yellow")
    sync_tyro_main(
        tyro.cli(cvc.VideoConfigUnion, description="Download videos, audio or generate word image.")
    )


if __name__ == "__main__":
    tyro_cli()



"""
Bili_UAS.scripts.video

This module provides the function to obtain video data and generate word cloud images.
"""


from __future__ import annotations
from typing import Union
import jieba
import wordcloud
import pandas as pd
from Bili_UAS.utils import video_utils as uvu
from numpy import typing as npt
from bilibili_api import Credential
from Bili_UAS.utils.config_utils import load_language_from_txt
from Bili_UAS.writer import log_writer as wlw
import enum
import os
from dataclasses import dataclass
import tyro


language: str = load_language_from_txt()


class WordCloudContent(enum.Enum):
    """
    Word Cloud Content Enumeration Class
    """
    REPLY = 1
    DANMU = 2
    BOTH = 3


async def word_cloud(video_id: Union[str, int],
                     credential: Union[Credential, None],
                     mode: WordCloudContent,
                     sec: bool,
                     mask: npt.NDArray,
                     log_file: str,
                     work_dir: str) -> None:
    """
    Obtain word cloud images of video replies or danmu.

    Args:
        video_id: video's aid or bvid
        credential: logon credentials
        mode: 0 represents only processing comments, 1 represents only processing danmu, and 2 represents both processing
        sec: whether to process secondary replies
        mask: word cloud mask, filling the white pixel with word clouds
        log_file: the log file
        work_dir: working directory
    """
    file_handler: wlw.Handler = wlw.Handler("file")
    file_handler.set_level("WARNING", "ERROR")
    file_handler.set_file(log_file)

    sys_handler: wlw.Handler = wlw.Handler("sys")
    sys_handler.set_level("INFO", "WARNING")

    log: wlw.Logger = wlw.Logger()
    log.add_config(file_handler)
    log.add_config(sys_handler)

    if isinstance(video_id, int):
        video = uvu.BiliVideo(log=log_file, aid=video_id, credential=credential, work_dir=work_dir)
    else:
        video = uvu.BiliVideo(log=log_file, bvid=video_id, credential=credential, work_dir=work_dir)

    if language == "en":
        log.info("Starting to generate word cloud image...")
    else:
        log.info("开始生成词云图片...")
    if mode == WordCloudContent.REPLY:
        save_path: str = os.path.join(video.work_dir, "reply_word_cloud.jpg")

        reply_content: str = ""
        await video.get_replies(sec=sec)
        await video.reply_robust_process()
        for elem in video.robust_replies:
            reply_content += elem.content
            reply_content += " "

        words: list[str] = jieba.lcut(reply_content)
        word_freq = pd.Series(words).value_counts()
        wc = wordcloud.WordCloud(font_path='PingFang.ttc', background_color='white', mask=mask)
        wc.generate_from_frequencies(word_freq)
        image = wc.to_image()
        image.save(save_path, quality=100)

    elif mode == WordCloudContent.DANMU:
        save_path: str = os.path.join(video.work_dir, "danmu_word_cloud.jpg")

        danmu_content: str = ""
        await video.get_danmu()
        for elem in video.danmu:
            danmu_content += elem.content
            danmu_content += " "

        words: list[str] = jieba.lcut(danmu_content)
        word_freq = pd.Series(words).value_counts()
        wc = wordcloud.WordCloud(font_path='PingFang.ttc', background_color='white', mask=mask)
        wc.generate_from_frequencies(word_freq)
        image = wc.to_image()
        image.save(save_path, quality=100)

    else:
        save_path: str = os.path.join(video.work_dir, "reply_and_danmu_word_cloud.jpg")

        content: str = ""
        await video.get_replies(sec=sec)
        await video.reply_robust_process()
        await video.get_danmu()
        for elem in video.robust_replies:
            content += elem.content
            content += " "
        for elem in video.danmu:
            content += elem.content
            content += " "

        words: list[str] = jieba.lcut(content)
        word_freq = pd.Series(words).value_counts()
        wc = wordcloud.WordCloud(font_path='PingFang.ttc', background_color='white', mask=mask)
        wc.generate_from_frequencies(word_freq)
        image = wc.to_image()
        image.save(save_path, quality=100)

    if language == "en":
        log.info(f"Word cloud image generated successfully! Saved in {save_path}.")
    else:
        log.info(f"词云图片生成成功！保存在 {save_path} 。")


# TODO: Effective Chinese sentiment analysis tool
# def _sentiment_analysis() -> None:
#     """
#     Conduct emotional analysis on video comments and barrage, and generate emotional statistical charts.
#     """
#     pass



"""

"""


from __future__ import annotations
import search_utils as su


async def collect_videos(keywords: list[str], log: str) -> list[str]:
    """
    Collect all videos matching keywords.
    """
    search: su.BiliSearch = su.BiliSearch(keywords, log)
    await search.search_video()
    result_list: list[str] = search.video_id
    return result_list



"""
Bili_UAS.scripts.log_in

This module provides the function to log in to Bilibili, save and read credentials.
"""


from __future__ import annotations
from bilibili_api import login, user, Credential
from bilibili_api import settings
from bilibili_api.exceptions import (CredentialNoBiliJctException, CredentialNoSessdataException,
                                     CredentialNoBuvid3Exception, CredentialNoDedeUserIDException)
import sys
import json
import os
from Bili_UAS.writer import log_writer as wlw
from Bili_UAS.utils.config_utils import load_language_from_txt
import enum
from typing import Union


language: str = load_language_from_txt()


class LoginMode(enum.Enum):
    """
    Login method Enumeration Class
    """
    QR = 1
    PASSWORD = 2
    VERIFICATION = 3
    PARAMETER = 4


async def load_credential_from_json(log_file: str) -> Union[Credential, None]:
    """
    Load credential from json file.

    Args:
        log_file: the log file
    """
    file_handler: wlw.Handler = wlw.Handler("file")
    file_handler.set_level("WARNING", "ERROR")
    file_handler.set_file(log_file)

    sys_handler: wlw.Handler = wlw.Handler("sys")
    sys_handler.set_level("INFO", "WARNING")

    log: wlw.Logger = wlw.Logger()
    log.add_config(file_handler)
    log.add_config(sys_handler)

    config_file: str = ".config.json"
    if not os.path.exists(config_file):
        if language == "en":
            log.warning("No historical login records found, using empty credential!")
        else:
            log.warning("未找到历史登录记录，使用空登录数据！")
        return None
    else:
        with open(config_file, "r") as f:
            credential_dict: dict = json.load(f)
            credential: Credential = Credential(sessdata=credential_dict["sessdata"],
                                                bili_jct=credential_dict["bili_jct"],
                                                buvid3=credential_dict["buvid3"],
                                                dedeuserid=credential_dict["dedeuserid"],
                                                ac_time_value=credential_dict["ac_time_value"])
        if language == "en":
            log.info("Historical login records found, using historical credential.")
        else:
            log.info("找到历史登录记录，使用历史登录数据。")
        return credential


async def save_credential_to_json(credential: Credential, log_file: str) -> None:
    """
    Save credential to json file.

    Args:
        credential: logon credentials
        log_file: the log file
    """
    file_handler: wlw.Handler = wlw.Handler("file")
    file_handler.set_level("WARNING", "ERROR")
    file_handler.set_file(log_file)

    sys_handler: wlw.Handler = wlw.Handler("sys")
    sys_handler.set_level("INFO", "WARNING")

    log: wlw.Logger = wlw.Logger()
    log.add_config(file_handler)
    log.add_config(sys_handler)

    config_file: str = ".config.json"
    credential_dict: dict = {"sessdata": credential.sessdata,
                             "bili_jct": credential.bili_jct,
                             "buvid3": credential.buvid3,
                             "dedeuserid": credential.dedeuserid,
                             "ac_time_value": credential.ac_time_value}
    with open(config_file, "w") as f:
        json.dump(credential_dict, f, indent=4)
    if language == "en":
        log.info("Credential saved successfully.")
    else:
        log.info("登录数据保存成功。")


async def save_credential_by_parm_to_json(sessdata: str,
                                          bili_jct: str,
                                          buvid3: str,
                                          dedeuserid: str,
                                          ac_time_value: str,
                                          log_file: str) -> None:
    """
    Save credential parameters to json file.

    Args:
        sessdata: credential sessdata
        bili_jct: credential bili_jct
        buvid3: credential buvid3
        dedeuserid: credential dedeuserid
        ac_time_value: credential ac_time_value
        log_file: the log file
    """
    file_handler: wlw.Handler = wlw.Handler("file")
    file_handler.set_level("WARNING", "ERROR")
    file_handler.set_file(log_file)

    sys_handler: wlw.Handler = wlw.Handler("sys")
    sys_handler.set_level("INFO", "WARNING")

    log: wlw.Logger = wlw.Logger()
    log.add_config(file_handler)
    log.add_config(sys_handler)

    config_file: str = ".config.json"
    credential_dict: dict = {"sessdata": sessdata,
                             "bili_jct": bili_jct,
                             "buvid3": buvid3,
                             "dedeuserid": dedeuserid,
                             "ac_time_value": ac_time_value}
    with open(config_file, "w") as f:
        json.dump(credential_dict, f, indent=4)
    if language == "en":
        log.info("Credential saved successfully.")
    else:
        log.info("登录数据保存成功。")


async def log_in_by_QR_code(log_file: str) -> bool:
    """
    Login to a Bilibili account by scanning QR code.

    Args:
        log_file: the log file

    Returns:
        True if login successfully, False otherwise.
    """
    file_handler: wlw.Handler = wlw.Handler("file")
    file_handler.set_level("WARNING", "ERROR")
    file_handler.set_file(log_file)

    sys_handler: wlw.Handler = wlw.Handler("sys")
    sys_handler.set_level("INFO", "WARNING")

    log: wlw.Logger = wlw.Logger()
    log.add_config(file_handler)
    log.add_config(sys_handler)

    if language == "en":
        log.info("Please scan the QR code.")
    else:
        log.info("请扫描二维码。")
    credential: Credential = login.login_with_qrcode()

    try:
        credential.raise_for_no_sessdata()
    except CredentialNoSessdataException:
        if language == "en":
            log.error("Login failed!")
        else:
            log.error("登录失败！")
        return False

    try:
        credential.raise_for_no_bili_jct()
    except CredentialNoBiliJctException:
        if language == "en":
            log.error("Login failed!")
        else:
            log.error("登录失败！")
        return False

    try:
        credential.raise_for_no_buvid3()
    except CredentialNoBuvid3Exception:
        if language == "en":
            log.error("Login failed!")
        else:
            log.error("登录失败！")
        return False

    try:
        credential.raise_for_no_dedeuserid()
    except CredentialNoDedeUserIDException:
        if language == "en":
            log.error("Login failed!")
        else:
            log.error("登录失败！")
        return False

    user_info: dict = await user.get_self_info(credential)
    if language == "en":
        log.info(f"Login successfully! Welcome {user_info['name']}!")
    else:
        log.info(f"登录成功！欢迎您，{user_info['name']}！")
    await save_credential_to_json(credential, log_file)
    return True


async def log_in_by_password(log_file: str) -> bool:
    """
    Login to a Bilibili account by entering the password.

    Args:
        log_file: the log file

    Returns:
        True if login successfully, False otherwise.
    """
    file_handler: wlw.Handler = wlw.Handler("file")
    file_handler.set_level("WARNING", "ERROR")
    file_handler.set_file(log_file)

    sys_handler: wlw.Handler = wlw.Handler("sys")
    sys_handler.set_level("INFO", "WARNING")

    log: wlw.Logger = wlw.Logger()
    log.add_config(file_handler)
    log.add_config(sys_handler)

    if language == "en":
        log.info("Please enter your username (phone number/email):\n")
    else:
        log.info("请输入您的用户名（手机号/邮箱）：\n")
    user_name: str = str(sys.stdin.readline())
    if language == "en":
        log.info("Please enter password:\n")
    else:
        log.info("请输入密码：\n")
    password: str = str(sys.stdin.readline())

    c = login.login_with_password(user_name, password)

    if isinstance(c, login.Check):
        if language == "en":
            log.error("Login failed!")
        else:
            log.error("登录失败！")
        return False
    else:
        credential: Credential = c

    try:
        credential.raise_for_no_sessdata()
    except CredentialNoSessdataException:
        if language == "en":
            log.error("Login failed!")
        else:
            log.error("登录失败！")
        return False

    try:
        credential.raise_for_no_bili_jct()
    except CredentialNoBiliJctException:
        if language == "en":
            log.error("Login failed!")
        else:
            log.error("登录失败！")
        return False

    try:
        credential.raise_for_no_buvid3()
    except CredentialNoBuvid3Exception:
        if language == "en":
            log.error("Login failed!")
        else:
            log.error("登录失败！")
        return False

    try:
        credential.raise_for_no_dedeuserid()
    except CredentialNoDedeUserIDException:
        if language == "en":
            log.error("Login failed!")
        else:
            log.error("登录失败！")
        return False

    user_info: dict = await user.get_self_info(credential)
    if language == "en":
        log.info(f"Login successfully! Welcome {user_info['name']}!")
    else:
        log.info(f"登录成功！欢迎您，{user_info['name']}！")
    await save_credential_to_json(credential, log_file)
    return True


async def log_in_by_verification_code(log_file: str) -> bool:
    """
    Login to a Bilibili account by using the verification code.

    Args:
        log_file: the log file

    Returns:
        True if login successfully, False otherwise.
    """
    file_handler: wlw.Handler = wlw.Handler("file")
    file_handler.set_level("WARNING", "ERROR")
    file_handler.set_file(log_file)

    sys_handler: wlw.Handler = wlw.Handler("sys")
    sys_handler.set_level("INFO", "WARNING")

    log: wlw.Logger = wlw.Logger()
    log.add_config(file_handler)
    log.add_config(sys_handler)

    settings.geetest_auto_open = False
    if language == "en":
        log.info("Please enter your phone number:\n")
    else:
        log.info("请输入您的手机号：\n")
    phone_number: str = str(sys.stdin.readline())
    if language == "en":
        log.info("Please wait for receiving the verification code...")
    else:
        log.info("请等待接收验证码...")
    login.send_sms(login.PhoneNumber(phone_number, country="+86"))
    if language == "en":
        log.info("Please enter the verification code:\n")
    else:
        log.info("请输入验证码：\n")
    code: str = str(sys.stdin.readline())
    c = login.login_with_sms(login.PhoneNumber(phone_number, country="+86"), code)

    if isinstance(c, login.Check):
        if language == "en":
            log.error("Login failed!")
        else:
            log.error("登录失败！")
        return False
    else:
        credential: Credential = c

    try:
        credential.raise_for_no_sessdata()
    except CredentialNoSessdataException:
        if language == "en":
            log.error("Login failed!")
        else:
            log.error("登录失败！")
        return False

    try:
        credential.raise_for_no_bili_jct()
    except CredentialNoBiliJctException:
        if language == "en":
            log.error("Login failed!")
        else:
            log.error("登录失败！")
        return False

    try:
        credential.raise_for_no_buvid3()
    except CredentialNoBuvid3Exception:
        if language == "en":
            log.error("Login failed!")
        else:
            log.error("登录失败！")
        return False

    try:
        credential.raise_for_no_dedeuserid()
    except CredentialNoDedeUserIDException:
        if language == "en":
            log.error("Login failed!")
        else:
            log.error("登录失败！")
        return False

    user_info: dict = await user.get_self_info(credential)
    if language == "en":
        log.info(f"Login successfully! Welcome {user_info['name']}!")
    else:
        log.info(f"登录成功！欢迎您，{user_info['name']}！")
    await save_credential_to_json(credential, log_file)
    return True


async def refresh_credential(credential: Credential, log_file: str) -> Credential:
    """
    Refresh the credential.

    Args:
        credential: logon credentials
        log_file: the log file

    Returns:
        refreshed credential
    """
    file_handler: wlw.Handler = wlw.Handler("file")
    file_handler.set_level("WARNING", "ERROR")
    file_handler.set_file(log_file)

    sys_handler: wlw.Handler = wlw.Handler("sys")
    sys_handler.set_level("INFO", "WARNING")

    log: wlw.Logger = wlw.Logger()
    log.add_config(file_handler)
    log.add_config(sys_handler)

    if credential.chcek_refresh():
        await credential.refresh()
        if language == "en":
            log.info("Credential refreshed successfully.")
        else:
            log.info("登录数据刷新成功。")
    else:
        if language == "en":
            log.warning("Credential do not need to be refreshed.")
        else:
            log.warning("登录数据不需要刷新。")

    return credential



"""
Bili_UAS.utils.config_utils
"""


from __future__ import annotations
import os
from Bili_UAS.writer import abnormal_monitor as wam


def load_language_from_txt() -> str:
    """
    Load a language from file.

    Returns:
        the language
    """
    config_file: str = ".language.txt"
    if not os.path.exists(config_file):
        return "en"
    else:
        with open(config_file, "r") as f:
            language: str = f.readline().removesuffix("\n")
        return language


async def load_work_dir_from_txt() -> str:
    """
    Load a working path from file.

    Returns:
        the work directory
    """
    config_file: str = ".work_dir.txt"
    language: str = load_language_from_txt()
    if not os.path.exists(config_file):
        if language == "en":
            raise wam.FileMissError("No historical working path found, please specify the working path.")
        else:
            raise wam.FileMissError("未找到记录工作路径的文件, 请指定工作路径.")
    else:
        with open(config_file, "r") as f:
            work_dir: str = f.readline().removesuffix("\n")
        if language == "en":
            print("INFO: Historical working path found, using historical working path.")
        else:
            print("INFO: 找到历史工作路径, 使用历史工作路径.")
        return work_dir


async def load_ffmpeg_path_from_txt() -> str:
    """
    Load the ffmpeg path.

    Returns:
        the path of ffmpeg
    """
    ffmpeg_file: str = ".ffmpeg.txt"
    language: str = load_language_from_txt()
    if not os.path.exists(ffmpeg_file):
        if language == "en":
            raise wam.FileMissError("No file found to record ffmpeg path, please specify the ffmpeg path.")
        else:
            raise wam.FileMissError("未找到记录ffmpeg路径的文件, 请指定ffmpeg路径.")
    else:
        with open(ffmpeg_file, "r") as f:
            ffmpeg: str = f.readline().removesuffix("\n")
        if language == "en":
            print("INFO: Historical ffmpeg path found, using historical ffmpeg path.")
        else:
            print("INFO: 找到历史ffmpeg路径, 使用历史ffmpeg路径.")
        return ffmpeg



"""
Bili_UAS.utils.user_utils

This module provides the BiliUser class, which is used to get user information and videos uploaded by the user.
"""


# data test_output path template: user_output/{user_id}/file_name: {fans_num.txt, guard_num.txt, charge_num.txt,
#                                                             address.xlsx, address_unreceived.txt}
# fans_num.txt content format: {time},{fans_num}
# guard_num.txt content format: {time},{guard_total_num},{governor_num},{supervisor_num},{captain_num}


from __future__ import annotations
import enum
from bilibili_api import user as bau, live as bal, sync
from .config_utils import load_language_from_txt
from Bili_UAS.writer import log_writer as wlw
from bilibili_api import Credential, session
import os
import time
from bilibili_api.exceptions import (CredentialNoBiliJctException, CredentialNoSessdataException,
                                     CredentialNoBuvid3Exception, CredentialNoDedeUserIDException)
import pandas as pd
from pandas import DataFrame
from typing import Union


language: str = load_language_from_txt()


class AddressProcessType(enum.Enum):
    """

    """
    SEND = 1
    RECEIVE = 2


async def _address_mag_simple_check(msg: list[str]) -> bool:
    """
    Check if it is address information.

    Args:
        msg: message list

    Returns:
        True if it is address information, otherwise False
    """
    if len(msg) != 3:
        return False
    if msg[0].split("：")[0] != "收件人":
        return False
    if msg[1].split("：")[0] != "电话":
        return False
    if not msg[1].split("：")[1].isdigit():
        return False
    if msg[2].split("：")[0] != "地址":
        return False
    return True


class BiliUser(bau.User, bal.LiveRoom):
    """
    Bilibili User Class
    """

    def __init__(self, uid: int, log: str, work_dir: str, credential: Union[Credential, None] = None) -> None:
        """
        Args:
            uid: user ID
            log: log file path
            credential: logon credentials
        """
        self.log_file: str = log
        self.log: Union[wlw.Logger, None] = None
        self.__set_log()

        bau.User.__init__(self, uid, credential)
        self.uid: int = uid
        self.room_id: Union[int, None] = None
        self.work_dir: Union[str, None] = None
        self.__live_init()

        self.video_id: list[str] = []

        self.fans_num_txt_file: Union[str, None] = None
        self.guard_num_txt_file: Union[str, None] = None
        self.charge_num_txt_file: Union[str, None] = None
        self.address_excel_file: Union[str, None] = None
        self.address_excel: Union[DataFrame, None] = None
        self.address_unreceived_txt_file: Union[str, None] = None

        self.__load_work_dir(work_dir)
        sync(self.__load_output_file())

    def __set_log(self) -> None:
        """
        Set up logs.
        """
        file_handler: wlw.Handler = wlw.Handler("file")
        file_handler.set_level("WARNING", "ERROR")
        file_handler.set_file(self.log_file)

        sys_handler: wlw.Handler = wlw.Handler("sys")
        sys_handler.set_level("INFO", "WARNING")

        self.log: wlw.Logger = wlw.Logger()
        self.log.add_config(file_handler)
        self.log.add_config(sys_handler)

    def __load_work_dir(self, work_dir: str) -> None:
        """
        Load the working directory.

        Args:
            work_dir: working directory
        """
        user_output_dir: str = os.path.join(work_dir, "user_output")
        if not os.path.exists(user_output_dir):
            os.mkdir(user_output_dir)

        self.work_dir: str = os.path.join(user_output_dir, str(self.uid))
        if not os.path.exists(self.work_dir):
            os.mkdir(self.work_dir)

    def __live_init(self) -> None:
        """
        Initialize live room.
        """
        live_info: dict = sync(self.get_live_info())
        if live_info['live_room'] is None:
            if language == "en":
                self.log.warning("The user has not opened a live streaming room!")
            else:
                self.log.warning("用户未开通直播间！")
            return
        self.room_id = live_info['live_room']['roomid']
        bal.LiveRoom.__init__(self, self.room_id, self.credential)

    async def __check_credentials(self) -> bool:
        """
        Check if the credentials are valid.

        Returns:
            True if the credentials are valid, False otherwise.
        """
        try:
            await self.credential.raise_for_no_sessdata()
        except CredentialNoSessdataException:
            if language == "en":
                self.log.warning("Credential missing SESSDATA!")
            else:
                self.log.warning("登录信息缺少SESSDATA！")
            return False

        try:
            await self.credential.raise_for_no_bili_jct()
        except CredentialNoBiliJctException:
            if language == "en":
                self.log.warning("Credential missing bili_jct!")
            else:
                self.log.warning("登录信息缺少bili_jct！")
            return False

        try:
            await self.credential.raise_for_no_buvid3()
        except CredentialNoBuvid3Exception:
            if language == "en":
                self.log.warning("Credential missing buvid3!")
            else:
                self.log.warning("登录信息缺少buvid3！")
            return False

        try:
            await self.credential.raise_for_no_dedeuserid()
        except CredentialNoDedeUserIDException:
            if language == "en":
                self.log.warning("Credential missing DedeUserID!")
            else:
                self.log.warning("登录信息缺少DedeUserID！")
            return False

        return True

    async def __load_output_file(self) -> None:
        """
        Load the test_output file.
        """
        self.fans_num_txt_file: str = os.path.join(self.work_dir, "fans_num.txt")
        if not os.path.exists(self.fans_num_txt_file):
            with open(self.fans_num_txt_file, "a") as f:
                f.write("query_time,fans_num\n\n")

        self.guard_num_txt_file: str = os.path.join(self.work_dir, "guard_num.txt")
        if not os.path.exists(self.guard_num_txt_file):
            with open(self.guard_num_txt_file, "a") as f:
                f.write("query_time,total_num,governor_num,supervisor_num,captain_num\n\n")

        self.charge_num_txt_file: str = os.path.join(self.work_dir, "charge_num.txt")
        if not os.path.exists(self.charge_num_txt_file):
            with open(self.charge_num_txt_file, "a") as f:
                f.write("query_time,charge_num\n\n")

        self.address_excel_file: str = os.path.join(self.work_dir, "address.xlsx")
        if not os.path.exists(self.address_excel_file):
            temp_excel: DataFrame = pd.DataFrame()
            temp_excel.to_excel(self.address_excel_file, index=False)
        self.address_excel: DataFrame = pd.read_excel(self.address_excel_file)

        self.address_unreceived_txt_file: str = os.path.join(self.work_dir, "address_unreceived.txt")
        if not os.path.exists(self.address_unreceived_txt_file):
            with open(self.address_unreceived_txt_file, "a") as f:
                f.write("uid,bili_name\n\n")

    @wlw.async_separate()
    async def get_upload_videos(self):
        """
        Get all videos uploaded by this user.
        """
        if language == "en":
            self.log.info(f"Start getting all videos uploaded by user {self.uid}...")
        else:
            self.log.info(f"开始获取用户 {self.uid} 上传的所有视频...")
        page: int = 1
        count: int = 0
        while True:
            video_data: dict = await self.get_videos(pn=page)
            if video_data['list']['vlist']:
                for video in video_data['list']['vlist']:
                    self.video_id.append(video['bvid'])
                    count += 1
            else:
                break
            if count >= video_data['page']['count']:
                break
            page += 1
        if language == "en":
            self.log.info(f"A total of {len(self.video_id)} videos were obtained.")
        else:
            self.log.info(f"共获取到 {len(self.video_id)} 个视频。")

    @wlw.async_separate()
    async def update_fans_number(self) -> None:
        """
        Update the number of fans.
        """
        now_time: int = int(time.time())
        if language == "en":
            self.log.info(f"Start getting the number of fans of user {self.uid}...")
        else:
            self.log.info(f"开始获取用户 {self.uid} 的粉丝数...")
        relation_info: dict = await self.get_relation_info()
        fans_num = relation_info['follower']
        if fans_num:
            with open(self.fans_num_txt_file, "a") as f:
                f.write(f"{time.strftime('%Y-%m-%d_%H-%M-%S', time.localtime(now_time))},{fans_num}\n")
        else:
            if language == "en":
                self.log.warning("Failed to get the number of fans!")
            else:
                self.log.warning("获取粉丝数失败！")

    @wlw.async_separate()
    async def update_guard_number(self) -> None:
        """
        Update the number of guards.
        """
        now_time: int = int(time.time())
        if language == "en":
            self.log.info(f"Start getting the number of guards of user {self.uid}...")
        else:
            self.log.info(f"开始获取用户 {self.uid} 的舰长数...")
        guard_info: dict = await self.get_dahanghai(page=1)
        total_page: int = guard_info['info']['page']  # TODO: need to check guard_info is not empty
        guard_num: int = guard_info['info']['num']
        governor_num: int = 0
        supervisor_num: int = 0
        flag: bool = True
        for elem in guard_info['top3']:
            if elem['guard_level'] == 1:
                governor_num += 1
            elif elem['guard_level'] == 2:
                supervisor_num += 1
            elif elem['guard_level'] == 3:
                flag = False
                break
        if flag:
            for i in range(1, total_page + 1):
                guard_info: dict = await self.get_dahanghai(page=i)
                for elem in guard_info['list']:
                    if elem['guard_level'] == 1:
                        governor_num += 1
                    elif elem['guard_level'] == 2:
                        supervisor_num += 1
                    elif elem['guard_level'] == 3:
                        break
        captain_num: int = guard_num - governor_num - supervisor_num
        with open(self.guard_num_txt_file, "a") as f:
            f.write(f"{time.strftime('%Y-%m-%d_%H-%M-%S', time.localtime(now_time))},{guard_num},{governor_num},"
                    f"{supervisor_num},{captain_num}\n")

    @wlw.async_separate()
    async def update_charge_number(self) -> None:
        """
        Update the number of charges.
        """
        now_time: int = int(time.time())
        if language == "en":
            self.log.info(f"Start getting the number of charging member of user {self.uid}...")
        else:
            self.log.info(f"开始获取用户 {self.uid} 的充电人数...")
        charge_info: dict = await self.get_elec_user_monthly()
        charge_num: int = charge_info['total_count']
        with open(self.charge_num_txt_file, "a") as f:
            f.write(f"{time.strftime('%Y-%m-%d_%H-%M-%S', time.localtime(now_time))},{charge_num}\n")

    @wlw.async_separate()
    async def guard_address_stat_send(self) -> None:
        """
        Send address statistics to the guard.
        """
        if language == "en":
            self.log.info(f"Start sending address statistics to the guard...")
        else:
            self.log.info(f"开始向舰长发送地址统计信息...")
        if not await self.__check_credentials():
            if language == "en":
                self.log.warning("Credential error, unable to perform address statistics!")
            else:
                self.log.warning("登录信息错误，无法进行地址统计！")
            return

        msg: str = "请按以下顺序输入地址信息：收件人，电话，地址。每项之间换行。示例：\n收件人：图图\n电话：123456\n地址：翻斗花园"
        guard_info: dict = await self.get_dahanghai(page=1)
        total_page: int = guard_info['info']['page']
        for elem in guard_info['top3']:
            target_uid: int = elem['uid']
            await session.send_msg(self.credential, target_uid, "1", msg)
        for i in range(1, total_page + 1):
            guard_info: dict = await self.get_dahanghai(page=i)  # TODO: need to check guard_info is not empty
            for elem in guard_info['list']:
                target_uid: int = elem['uid']
                await session.send_msg(self.credential, target_uid, "1", msg)
        if language == "en":
            self.log.info(f"Send successfully.")
        else:
            self.log.info(f"发送成功。")

    @wlw.async_separate()
    async def guard_address_stat_receive(self) -> None:
        """
        Receive address statistics from the guard.
        """
        if language == "en":
            self.log.info("Start receiving address statistics to the guard")
        else:
            self.log.info("开始接收舰长的地址统计信息")
        if not await self.__check_credentials():
            if language == "en":
                self.log.warning("Credential error, unable to receive address statistics!")
            else:
                self.log.warning("登录信息错误，无法接收地址信息！")
            return

        receive_flag: bool = False
        count: int = 0
        guard_info: dict = await self.get_dahanghai(page=1)  # TODO: need to check guard_info is not empty
        total_page: int = guard_info['info']['page']
        guard_num: int = guard_info['info']['num']
        for elem in guard_info['top3']:
            target_uid: int = elem['uid']
            target_name: str = elem['username']
            receive_info = await session.fetch_session_msgs(target_uid, self.credential)
            msg_list: list[dict] = receive_info['messages']
            for msg_dict in msg_list:
                content: list[str] = eval(msg_dict['content'])['content'].split("\n")
                temp_flag: bool = await _address_mag_simple_check(content)
                if temp_flag:
                    line: DataFrame = pd.DataFrame({"uid": target_uid,
                                                    "bili_name": target_name,
                                                    "收件人": content[0].split("：")[1],
                                                    "电话": content[1].split("：")[1],
                                                    "地址": content[2].split("：")[1]},
                                                   index=[0])
                    self.address_excel = pd.concat([self.address_excel, line], ignore_index=True, axis=0)
                    self.address_excel.to_excel(self.address_excel_file, index=False)
                    self.address_excel = pd.read_excel(self.address_excel_file)
                    count += 1
                    receive_flag = True
                    break
            if not receive_flag:
                with open(self.address_unreceived_txt_file, "a") as f:
                    f.write(f"{target_uid},{target_name}\n")
            receive_flag = False
        for i in range(1, total_page + 1):
            guard_info: dict = await self.get_dahanghai(page=i)
            for elem in guard_info['list']:
                target_uid: int = elem['uid']
                target_name: str = elem['username']
                receive_info = await session.fetch_session_msgs(target_uid, self.credential)
                msg_list: list[dict] = receive_info['messages']
                for msg_dict in msg_list:
                    content: list[str] = eval(msg_dict['content'])['content'].split("\n")
                    temp_flag: bool = await _address_mag_simple_check(content)
                    if temp_flag:
                        line: DataFrame = pd.DataFrame({"uid": target_uid,
                                                        "bili_name": target_name,
                                                        "收件人": content[0].split("：")[1],
                                                        "电话": content[1].split("：")[1],
                                                        "地址": content[2].split("：")[1]},
                                                       index=[0])
                        self.address_excel = pd.concat([self.address_excel, line], ignore_index=True, axis=0)
                        self.address_excel.to_excel(self.address_excel_file, index=False)
                        self.address_excel = pd.read_excel(self.address_excel_file)
                        count += 1
                        receive_flag = True
                        break
                if not receive_flag:
                    with open(self.address_unreceived_txt_file, "a") as f:
                        f.write(f"{target_uid},{target_name}\n")
                receive_flag = False
        if count == guard_num:
            if language == "en":
                self.log.info(f"Receive successfully. All addresses have been received, but may still contain invalid "
                              f"addresses that require manual inspection.")
            else:
                self.log.info(f"接收成功。已经接收到所有地址，但仍可能需要人工检查是否含有无效地址。")
        else:
            if language == "en":
                self.log.warning(f"Receive successfully. But only {count} address messages were received in total. "
                                 f"Guard whose address is not receives are recorded in {self.address_unreceived_txt_file}.")
            else:
                self.log.warning(f"接收成功。但总共只接收到了{count}个地址信息。"
                                 f"未接收到地址信息的舰长记录在 {self.address_unreceived_txt_file} 中。")



"""
Bili_UAS.cli.video_cli

This module provides video command line interface configuration.
"""


from dataclasses import dataclass
from typing import Union, Literal
import tyro


@dataclass
class BiliVideoConfigWordCloud(object):
    """
    Bilibili Video Configuration Class: Word Cloud.
    """
    video_id: Union[str, int, None] = None
    """video's aid or bvid"""
    mode: Literal[1, 2, 3] = 1
    """word cloud content, 1 represents comments, 2 represents barrage, and 3 represents both"""
    sec: bool = True
    """whether to process secondary replies"""
    mask: Union[str, None] = None
    """word cloud mask, filling the white pixel with word clouds"""


@dataclass
class BiliVideoConfigDownload(object):
    """
    Bilibili Video Configuration Class: Download.
    """
    video_id: Union[str, int, None] = None
    """video's aid or bvid"""
    mode: Literal[1, 2] = 1
    """video download type, 1 represents video and 2 represents audio"""


mode_configs: dict[str, Union[BiliVideoConfigWordCloud, BiliVideoConfigDownload]] = {}

descriptions: dict[str, str] = {
    "word_cloud": "Generate word cloud images of video replies or danmu.",
    "download": "Download video or audio."
}

mode_configs["word_cloud"] = BiliVideoConfigWordCloud()
mode_configs["download"] = BiliVideoConfigDownload()

VideoConfigUnion = tyro.conf.SuppressFixed[
    tyro.conf.FlagConversionOff[
        tyro.extras.subcommand_type_from_defaults(defaults=mode_configs, descriptions=descriptions)
    ]
]



"""
Bili_UAS.utils.utils

Some small classes, recording danmu, reply, and other information.
"""


from __future__ import annotations
import pandas as pd
from bilibili_api import Danmaku
from pandas import DataFrame
from typing import Union


class BiliVideoReply(object):
    """
    Bilibili video reply class.
    """

    def __init__(self, reply_data: dict, log: str) -> None:
        """
        Args:
             reply_data: dictionary for saving reply information
             log: log file path
        """
        self.rpid: int = reply_data['rpid']  # Reply id
        self.mid: int = reply_data['mid']  # Reply publisher id
        self.sec_replies_num: int = reply_data['count']  # Number of secondary replies
        self.replies_num: int = reply_data['rcount']  # Number of replies
        self.content: str = reply_data['content']['message']  # Reply content
        self.like: str = reply_data['like']
        self.log_file: str = log


class BiliVideoDanmu(object):
    """
    Bilibili Video Danmu Class.
    """
    def __init__(self, danmu_data: Danmaku, log: str) -> None:
        """
        Args:
            danmu_data: danmu protobuf data
            log: the log file
        """
        self.dmid: int = danmu_data.id_  # Danmu id
        # Type of danmu. 1 2 3: Normal danmu 4: Bottom danmu 5: Top danmu 6: Reverse danmu
        # 7: Advanced danmu 8: Code danmu 9: BAS danmu
        self.mode: int = danmu_data.mode
        # Danmu pool. 0: Normal pool 1: Subtitle pool 2: Special pool (code/BAS barrage)
        self.pool: int = danmu_data.pool
        self.content: str = danmu_data.text  # Danmu content
        # Range: [0-10]. The higher the value, the higher the weight. Used for intelligent shielding.
        self.weight: int = danmu_data.weight
        self.time: int = int(danmu_data.send_time)  # Unix timestamp of danmu sending time
        self.log_file: str = log


class BiliVideoTag(object):
    """
    Bilibili Video Tag Class.
    """

    def __init__(self, tag_data: dict, log: str) -> None:
        """
        Args:
             tag_data: dictionary for saving tag information
             log: the log file
        """
        self.tag_id: int = tag_data['tag_id']  # Tag id
        self.tag_name: str = tag_data['tag_name']
        self.use_num: int = tag_data['count']['use']  # Number of videos with this tag added
        self.follow_num: int = tag_data['count']['atten']  # Number of users following this tag
        self.log_file: str = log


class BiliLiveDanmu(object):
    """
    Bilibili live danmu class.
    """

    def __init__(self, log: str):
        """
        Args:
            log: log file path
        """
        # The API return information of the danmu here is stored in the form of a list, so the readability of
        # the parsing is not high, and there may be bugs in the future.
        # But this is Bilibili's problem.
        self.log_file: str = log
        self.content: Union[str, None] = None
        self.time: Union[int, None] = None
        self.user_uid: Union[int, None] = None

    async def load_from_api(self, data: dict) -> None:
        """
        Load danmu information from API data.

        Args:
            data: dictionary for storing live-streaming danmu information
        """
        self.content: str = data['data']['info'][1]
        self.time: int = int(data['data']['info'][0][4] / 1000)
        self.user_uid: int = data['data']['info'][2][0]

    async def to_excel(self, excel_file: str, excel: DataFrame) -> None:
        """
        Write danmu information to excel file.

        Args:
            excel_file: excel file path
            excel: excel file
        """
        line: DataFrame = pd.DataFrame({"user_uid": self.user_uid,
                                        "content": self.content,
                                        "time": self.time},
                                       index=[0])
        excel = pd.concat([excel, line], ignore_index=True, axis=0)
        excel.to_excel(excel_file, index=False)

    async def load_from_excel(self, data: dict) -> None:
        """
        Load danmu information from excel file.

        Args:
            data: dictionary for storing live-streaming danmu information
        """
        self.content: str = data['content']
        self.time: int = int(data['time'])
        self.user_uid: int = int(data['user_uid'])


class BiliLiveGift(object):
    """
    Bilibili live gift class.
    """

    def __init__(self, log: str):
        """
        Args:
            log: log file path
        """
        self.log_file: str = log
        self.gift_name: Union[str, None] = None
        self.gift_id: Union[int, None] = None
        self.number: Union[int, None] = None
        self.price: Union[float, None] = None  # unit: RMB
        self.time: Union[int, None] = None
        self.user_uid: Union[int, None] = None

    async def load_from_api(self, data: dict) -> None:
        """
        Load gift information from API data.

        Args:
            data: dictionary for storing live-streaming gift information
        """
        self.gift_name: str = data['data']['data']['giftName']
        self.gift_id: int = data['data']['data']['giftId']
        self.number: int = data['data']['data']['num']
        self.price: float = data['data']['data']['total_coin'] * 0.001

    async def to_excel(self, excel_file: str, excel: DataFrame) -> None:
        """
        Write gift information to excel file.

        Args:
            excel_file: excel file path
            excel: excel file
        """
        line: DataFrame = pd.DataFrame({"user_uid": self.user_uid,
                                        "gift_name": self.gift_name,
                                        "gift_id": self.gift_id,
                                        "number": self.number,
                                        "total_price": self.price,
                                        "time": self.time},
                                       index=[0])
        excel = pd.concat([excel, line], ignore_index=True, axis=0)
        excel.to_excel(excel_file, index=False)

    async def load_from_excel(self, data: dict) -> None:
        """
        Load gift information from excel file.

        Args:
            data: dictionary for storing live-streaming gift information
        """
        self.gift_name: str = data['gift_name']
        self.gift_id: int = int(data['gift_id'])
        self.number: int = int(data['number'])
        self.price: float = float(data['total_price'])
        self.time: int = int(data['time'])
        self.user_uid: int = int(data['user_uid'])


class BiliLiveSC(object):
    """
    Bilibili live super chat class.
    """

    def __init__(self, log: str):
        """
        Args:
            log: log file path
        """
        self.log_file: str = log
        self.content: Union[str, None] = None
        self.price: Union[float, None] = None  # unit: RMB
        self.time: Union[int, None] = None
        self.user_uid: Union[int, None] = None
        self.gift_id: Union[int, None] = None

    async def load_from_api(self, data: dict) -> None:
        """
        Load super chat information from API data.

        Args:
            data: dictionary for storing live-streaming super chat information
        """
        self.content: str = data['data']['data']['message']
        self.price: float = data['data']['data']['price']
        self.time: int = int(data['data']['data']['start_time'])
        self.user_uid: int = data['data']['data']['uid']
        self.gift_id: int = data['data']['data']['gift']['gift_id']

    async def to_excel(self, excel_file: str, excel: DataFrame) -> None:
        """
        Write super chat information to excel file.

        Args:
            excel_file: excel file path
            excel: excel file
        """
        line: DataFrame = pd.DataFrame({"user_uid": self.user_uid,
                                        "content": self.content,
                                        "price": self.price,
                                        "time": self.time,
                                        "gift_id": self.gift_id},
                                       index=[0])
        excel = pd.concat([excel, line], ignore_index=True, axis=0)
        excel.to_excel(excel_file, index=False)

    async def load_from_excel(self, data: dict) -> None:
        """
        Load super chat information from excel file.

        Args:
            data: dictionary for storing live-streaming super chat information
        """
        self.content: str = data['content']
        self.price: float = float(data['price'])
        self.time: int = int(data['time'])
        self.user_uid: int = int(data['user_uid'])
        self.gift_id: int = int(data['gift_id'])


class BiliLiveGuard(object):
    """
    Bilibili live guard class.
    """

    def __init__(self, log: str):
        """
        Args:
            log: log file path
        """
        self.log_file: str = log
        self.guard_level: Union[int, None] = None
        self.gift_id: Union[int, None] = None
        self.guard_name: Union[str, None] = None
        self.time: Union[int, None] = None
        self.price: Union[float, None] = None  # unit: RMB
        self.user_uid: Union[int, None] = None

    async def load_from_api(self, data: dict) -> None:
        """
        Load guard information from API data.

        Args:
            data: dictionary for storing live-streaming guard information
        """
        self.guard_level: int = data['data']['data']['guard_level']
        self.gift_id: int = data['data']['data']['gift_id']
        self.guard_name: str = data['data']['data']['gift_name']
        self.time: int = int(data['data']['data']['start_time'])
        self.price: float = data['data']['data']['price'] * 0.001
        self.user_uid: int = data['data']['data']['uid']

    async def to_excel(self, excel_file: str, excel: DataFrame) -> None:
        """
        Write guard information to excel file.

        Args:
            excel_file: excel file path
            excel: excel file
        """
        line: DataFrame = pd.DataFrame({"user_uid": self.user_uid,
                                        "guard_level": self.guard_level,
                                        "gift_id": self.gift_id,
                                        "guard_name": self.guard_name,
                                        "time": self.time,
                                        "price": self.price},
                                       index=[0])
        excel = pd.concat([excel, line], ignore_index=True, axis=0)
        excel.to_excel(excel_file, index=False)

    async def load_from_excel(self, data: dict) -> None:
        """
        Load guard information from excel file.

        Args:
            data: dictionary for storing live-streaming guard information
        """
        self.guard_level: int = int(data['guard_level'])
        self.gift_id: int = int(data['gift_id'])
        self.guard_name: str = data['guard_name']
        self.time: int = int(data['time'])
        self.price: float = float(data['price'])
        self.user_uid: int = int(data['user_uid'])



from setuptools import setup

with open("README.md", "r", encoding="utf-8") as f:
    long_description = f.read()

requirements = [
    'bilibili-api-python~=15.5.1',
    'setuptools~=68.0.0',
    'wordcloud~=1.9.2',
    'jieba~=0.42.1',
    'scipy~=1.11.1',
    'numpy~=1.25.1',
    'tyro~=0.5.4',
    'pandas~=2.0.3',
    'tqdm~=4.65.0',
    'matplotlib~=3.7.2',
    'asyncio~=3.4.3',
    'apscheduler~=3.10.1',
    'requests~=2.31.0',
    'httpx~=0.24.1',
    'openpyxl~=3.1.2']

setup(
    name='bili-uas',
    version='0.2.3',
    license='GPLv3',
    author='jhzg',
    author_email='jhzg02200059@163.com',
    url='https://github.com/jhzgjhzg/BiliBili-UP-Auxiliary-System',
    description='Assist up in personal, live, and video data analysis and prediction.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    keywords=["BiliBili", "auxiliary", "analysis", "live", "video", "word_cloud", "monitor"],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Topic :: Software Development :: Build Tools',
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)"
    ],
    packages=['Bili_UAS',
              'Bili_UAS.cli',
              'Bili_UAS.utils',
              'Bili_UAS.writer',
              'Bili_UAS.scripts'],
    package_data={'': ['*.txt', '*.md', 'LICENSE', 'setup.py', 'requirements.txt', '.gitignore']},
    exclude_package_data={'': ['test/*.py', 'test']},
    include_package_data=True,
    python_requires='>=3.9',
    install_requires=requirements,
    entry_points={
        'console_scripts': [
            'bili-config = Bili_UAS.bili_config:tyro_cli',
            'bili-live = Bili_UAS.bili_live:tyro_cli',
            'bili-login = Bili_UAS.bili_login:tyro_cli',
            'bili-video = Bili_UAS.bili_video:tyro_cli',
            'bili-user = Bili_UAS.bili_user:tyro_cli'
        ],
    }
)



"""
Bili_UAS.bili_login

This module provides a command line interface for logining to bilibili.
"""


from __future__ import annotations
from Bili_UAS.utils import config_utils as ucu
from Bili_UAS.scripts import log_in as sli
from bilibili_api import sync
import os
from typing import Literal, Union
import tyro
from Bili_UAS.writer import log_writer as wlw, abnormal_monitor as wam


def sync_tyro_main(mode: Literal[1, 2, 3, 4] = 1,
                   sessdata: Union[str, None] = None,
                   bili_jct: Union[str, None] = None,
                   buvid3: Union[str, None] = None,
                   dedeuserid: Union[str, None] = None,
                   ac_time_value: Union[str, None] = None) -> None:
    """
    Login to Bilibili.

    Args:
        mode: login mode, 1 for scanning QR code login, 2 for password login, 3 for verification code login, 4 for
              specifying credential parameters
        sessdata: credential sessdata
        bili_jct: credential bili_jct
        buvid3: credential buvid3
        dedeuserid: credential dedeuserid
        ac_time_value: credential ac_time_value
    """
    language: str = ucu.load_language_from_txt()
    work_dir: str = sync(ucu.load_work_dir_from_txt())
    log_output: str = os.path.join(work_dir, "log")
    log_file: str = os.path.join(log_output, "login_log.txt")

    file_handler: wlw.Handler = wlw.Handler("file")
    file_handler.set_level("WARNING", "ERROR")
    file_handler.set_file(log_file)

    sys_handler: wlw.Handler = wlw.Handler("sys")
    sys_handler.set_level("INFO", "WARNING", "ERROR")

    log: wlw.Logger = wlw.Logger()
    log.add_config(file_handler)
    log.add_config(sys_handler)

    m = sli.LoginMode(mode)

    if m == sli.LoginMode.PARAMETER:
        if sessdata is not None and bili_jct is not None and buvid3 is not None \
                and dedeuserid is not None and ac_time_value is not None:
            pass
        else:
            if language == "en":
                raise wam.ParameterInputError("Select the login method with the specified parameters,"
                                              "but the input parameters are missing!")
            else:
                raise wam.ParameterInputError("选择指定参数登录方式，但输入参数缺失！")

    if m == sli.LoginMode.QR:
        flag = sync(sli.log_in_by_QR_code(log_file))
    elif m == sli.LoginMode.PASSWORD:
        flag = sync(sli.log_in_by_password(log_file))
    elif m == sli.LoginMode.VERIFICATION:
        flag = sync(sli.log_in_by_verification_code(log_file))
    else:
        flag = sync(sli.save_credential_by_parm_to_json(sessdata, bili_jct, buvid3,
                                                        dedeuserid, ac_time_value, log_file))

    if not flag:
        if language == "en":
            log.error("Login failed, please try logging in again! (priority is to scan QR code or specify parameters)")
        else:
            log.error("登录失败，请重试登录！（优先扫描二维码或指定参数）")


def tyro_cli() -> None:
    """
    Command line interface
    """
    tyro.extras.set_accent_color("bright_yellow")
    tyro.cli(sync_tyro_main)


if __name__ == "__main__":
    tyro_cli()
